"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const events_1 = require("events");
const worker_threads_1 = require("worker_threads");
const crypto = require("crypto");
const eventBus = new events_1.EventEmitter();
const workerPool = [];
const workerPoolSetting = {
    maxWorkerThreads: 4,
};
/**
 * @param setting  Custom setting
 */
function init(setting = {}) {
    workerPoolSetting.maxWorkerThreads = setting.maxWorkerThreads || workerPoolSetting.maxWorkerThreads;
    if (workerPoolSetting.maxWorkerThreads <= 0) {
        throw new Error('maxWorkerThreads should not <= 0!');
    }
    for (let i = 0; i < workerPoolSetting.maxWorkerThreads; i++) {
        const w = new worker_threads_1.Worker(path.resolve(__dirname, './worker.js'));
        w.on('message', (response) => {
            const { uid } = response;
            eventBus.emit(`uid-${uid}`, response);
        });
        workerPool.push({
            id: i,
            instance: w,
            active: 0,
            achieved: 0,
        });
    }
}
exports.init = init;
/**
 * @param handler  routine function
 * @param context  variables defined in context
 *
 * @returns        handler function result
 */
async function go(handler, context = {}) {
    let definition = '';
    for (const key in context) {
        if (!context.hasOwnProperty(key) || !context[key])
            continue;
        definition += `const ${key} = ${JSON.stringify(context[key])};\n`;
    }
    const routineStr = `
async () => {
  ${definition}
  return await (${handler.toString()})()
}
`;
    const worker = selectWorker();
    const uid = genUid();
    worker.active += 1;
    worker.instance.postMessage({
        uid,
        workerId: worker.id,
        handler: routineStr,
    });
    return new Promise((resolve, reject) => {
        eventBus.once(`uid-${uid}`, (resp) => {
            worker.active -= 1;
            worker.achieved += 1;
            if (resp.error) {
                reject(resp.error);
            }
            else {
                resolve(resp.data);
            }
        });
    });
}
exports.go = go;
/**
 * Shutdown worker threads pool
 */
async function shutdown() {
    for (const { instance } of workerPool) {
        await instance.terminate();
    }
}
exports.shutdown = shutdown;
function selectWorker() {
    workerPool.sort((a, b) => (a.active - b.active));
    return workerPool[0];
}
function genUid() {
    return crypto.randomBytes(12).toString('hex');
}
//# sourceMappingURL=index.js.map